@@ -60,7 +60,6 @@ from asyncdb.providers.sql import (
 max_cached_statement_lifetime = 600
 max_cacheable_statement_size = 1024 * 15
 
-
 class pgPool(BasePool):
     _max_queries = 100
     _dsn = "postgres://{user}:{password}@{host}:{port}/{database}"
@@ -72,8 +71,12 @@ class pgPool(BasePool):
     application_name = 'Navigator'
 
     def __init__(self, dsn="", loop=None, params={}, **kwargs):
-        super(pgPool,
-              self).__init__(dsn=dsn, loop=loop, params=params, **kwargs)
+        super(pgPool, self).__init__(
+            dsn=dsn,
+            loop=loop,
+            params=params,
+            **kwargs
+        )
         if "server_settings" in kwargs:
             self._server_settings = kwargs["server_settings"]
         if 'application_name' in self._server_settings:
@@ -184,6 +187,7 @@ class pgPool(BasePool):
                 command_timeout=self._timeout,
                 init=self.init_connection,
                 setup=self.setup_connection,
+                loop=self._loop,
                 max_cached_statement_lifetime=max_cached_statement_lifetime,
                 max_cacheable_statement_size=max_cacheable_statement_size,
                 server_settings=server_settings,
@@ -233,20 +237,22 @@ class pgPool(BasePool):
         try:
             self._connection = await self._pool.acquire()
         except TooManyConnectionsError as err:
-            print("Too Many Connections Error: {}".format(str(err)))
+            self._logger.error("Too Many Connections Error: {}".format(str(err)))
             return False
         except ConnectionDoesNotExistError as err:
-            print("Connection Error: {}".format(str(err)))
+            self._logger.error("Connection Error: {}".format(str(err)))
             return False
         except InternalClientError as err:
-            print("Internal Error: {}".format(str(err)))
+            self._logger.error("Internal Error: {}".format(str(err)))
             return False
         except InterfaceError as err:
-            print("Interface Error: {}".format(str(err)))
+            self._logger.error("Interface Error: {}".format(str(err)))
             return False
         except InterfaceWarning as err:
-            print("Interface Warning: {}".format(str(err)))
+            self._logger.error("Interface Warning: {}".format(str(err)))
             return False
+        except Exception as err:
+            self._logger.error('Unknown Error on Acquire: {}'.format(str(err)))
         if self._connection:
             db = pg(pool=self)
             db.set_connection(self._connection)
@@ -271,7 +277,7 @@ class pgPool(BasePool):
         except InterfaceError as err:
             raise ProviderError("Release Interface Error: {}".format(str(err)))
         except InternalClientError as err:
-            logging.debug(
+            self._logger.debug(
                 "Connection already released, PoolConnectionHolder.release() called on a free connection holder"
             )
             # print("PoolConnectionHolder.release() called on a free connection holder")
@@ -340,7 +346,7 @@ class pgPool(BasePool):
             try:
                 await asyncio.wait_for(
                     close,
-                    timeout=timeout,
+                    timeout=2,
                     loop=self._loop
                 )
             except asyncio.exceptions.TimeoutError as err:
@@ -397,8 +403,11 @@ class pg(SQLProvider):
     _server_settings = {}
     application_name: str = 'Navigator'
 
-    def __init__(self, dsn: str = '', loop = None, params={}, **kwargs):
+    def __init__(self, dsn='', loop=None, params={}, pool=None, **kwargs):
         super(pg, self).__init__(dsn=dsn, loop=loop, params=params, **kwargs)
+        if pool:
+            self._pool = pool
+            self._loop = self._pool.get_event_loop()
         if "server_settings" in kwargs:
             self._server_settings = kwargs["server_settings"]
         if 'application_name' in self._server_settings:
