@@ -4,11 +4,14 @@ from .encoders import (
     BaseEncoder
 )
 from .functions import (
+    colors,
     SafeDict,
     is_pgconstant,
     is_udf,
+    _escapeString,
+    Msg
 )
 
 __all__ = [
-    "SafeDict", "DefaultEncoder", "BaseEncoder", "EnumEncoder", "is_pgconstant", "is_udf"
+    "colors", "SafeDict", "DefaultEncoder", "BaseEncoder", "EnumEncoder", "is_pgconstant", "is_udf", "_escapeString", "Msg"
 ]
